#ifndef EXT4_H
#define EXT4_H

#include <sys/types.h>

#include <stdint.h>

#define EXT4_NDIR_BLOCKS	12
#define EXT4_IND_BLOCK		EXT4_NDIR_BLOCKS
#define EXT4_DIND_BLOCK		(EXT4_IND_BLOCK + 1)
#define EXT4_TIND_BLOCK		(EXT4_DIND_BLOCK + 1)
#define EXT4_N_BLOCKS		(EXT4_TIND_BLOCK + 1)

#define EXT4_MAGIC		0xf30a

/* important flags */
#define EXT4_EXTENTS_FL		0x80000 /* inode uses extents */
#define EXT4_HUGE_FILE_FL	0x40000 /* set to each huge file */

#define EXT4_UNHANDLED_FLAGS \
    (EXT4_HUGE_FILE_FL)

/* inode offsets */
#define EXT4_IOFF_MODE		0	/* 16 */
#define EXT4_IOFF_UID		2	/* 16 */
#define EXT4_IOFF_SZLO		4	/* 32 */
#define EXT4_IOFF_GID		24	/* 16 */
#define EXT4_IOFF_NUMBLOCKS	28	/* 32 */
#define EXT4_IOFF_FLAGS		32	/* 32 */
#define EXT4_IOFF_BLOCKS	40	/* 15*4 bytes */
#define EXT4_IOFF_SZHI		108	/* 32 */

/* extent header offsets */
#define EXT4_SZ_EXTHDR		12
#define EXT4_EHOFF_MAGIC	0	/* 16 */
#define EXT4_EHOFF_COUNT	2	/* 16 */
#define EXT4_EHOFF_DEPTH	6	/* 16 */

/* extent index offsets */
#define EXT4_SZ_EXTIDX		12
#define EXT4_EIOFF_LOGIC	0	/* 32 */
#define EXT4_EIOFF_LEAFLO	4	/* 32 */
#define EXT4_EIOFF_LEAFHI	8	/* 16 */

/* extent offsets */
#define EXT4_SZ_EXTENT		12
#define EXT4_EOFF_LOGIC		0	/* 32 */
#define EXT4_EOFF_COUNT		4	/* 16 */
#define EXT4_EOFF_BLKHI		6	/* 16 */
#define EXT4_EOFF_BLKLO		8	/* 32 */

/* directory entry offsets */
#define EXT4_DOFF_INODE		0	/* 32 */
#define EXT4_DOFF_RECSZ		4	/* 16 */
#define EXT4_DOFF_NAMELEN	6	/* 8 */
#define EXT4_DOFF_TYPE		7	/* 8 */
#define EXT4_DOFF_NAME		8

/* filesystem specific values */
#define INODES_PER_GROUP	0x4020 /* 16416 */
#define BLOCKS_PER_GROUP	0x8000 /* 32768 */
#define SZ_INODE		0x80 /* 128B */
#define SZ_BLOCK		0x1000 /* 4kB */
#define FIRST_INODE		0xb /* 11 */
#define SZ_GROUP (SZ_BLOCK * BLOCKS_PER_GROUP) /* 128MB */
#define ADDRS_PER_BLOCK		(SZ_BLOCK / 4) /* 1024 */

off_t		get_inode_offset(uint32_t);

#endif
