#include <sys/types.h>
#include <sys/stat.h>

#include <fcntl.h>
#include <stdio.h>
#include <unistd.h>

#include "log.h"

int main(int argc, char **argv)
{
	int			fd;
	struct log_entry	*log;
	const char		*path;
	const char		*dest;
	uint16_t		uid;
	uint16_t		gid;
	uint16_t		perm;
	uint8_t			pretend;
	
	pretend = 0;

	if (argc != 2) {
		printf("usage: %s LOG_FILE\n", *argv);
		return 1;
	}

	if ((fd = open(argv[1], O_RDONLY)) == -1) {
		perror("main(): open()");
		return 1;
	}

	for (;;) {
		if (!(log = log_read(fd))) {
			fprintf(stderr, "main(): failed to read log entry\n");
			close(fd);
			return 1;
		}

		path = log_getpath(log);
		printf("%s\n", path);

		dest = log_getlink(log);
		uid = log_getuid(log);
		gid = log_getgid(log);
		perm = log_getperm(log);

		if (uid == LOG_UNDEFINED && gid != LOG_UNDEFINED) {
			printf("warn: only GID was specified, "
			    "skipping chown\n");
			gid = LOG_UNDEFINED;
		} else if (uid != LOG_UNDEFINED && gid == LOG_UNDEFINED) {
			printf("warn: only UID was specified, "
			    "using GID=0\n");
			gid = 0;
		}

		if (dest) {
			if (pretend) {
				printf("LINK %s\n", dest);
			} else if (symlink(dest, path) == -1) {
				perror("main(): symlink()");
				/* no sense trying a chmod() */
				log_free(log);
				continue;
			}
		} else if (perm != LOG_UNDEFINED) {
			if (pretend) {
				printf("CHMOD 0%03o\n", perm);
			} else if (chmod(path, perm) == -1) {
				perror("main(): chmod()");
			}
		}

		/* either both or neither are defined */
		if (uid != LOG_UNDEFINED) {
			if (pretend) {
				printf("CHOWN %d:%d\n", uid, gid);
			} else if (chown(path, uid, gid) == -1) {
				perror("main(): chown()");
			}
		}

		log_free(log);
	}

	close(fd);
	return 0;
}
