#ifndef TYPES_H
#define TYPES_H

#include <stdint.h>

#include "ext4.h"

/* the values are for directory entries, and not the mode */
enum file_type {
	FT_UNKNOWN = 0,
	FT_REG = 1,
	FT_DIR = 2,
	FT_CHRDEV = 3,
	FT_BLKDEV = 4,
	FT_FIFO = 5,
	FT_SOCK = 6,
	FT_SYMLINK = 7
};

enum block_type {
	BLK_UNDEFINED = 0,
	BLK_EXTENT,
	BLK_PLAIN,
	BLK_NONE
};

struct dir_entry {
	char			*name;
	uint32_t		inode;
	enum file_type		type;
#ifdef DEBUG
	size_t			offset;
#endif

	struct dir_entry	*next;
};

struct dir {
	struct dir_entry	*head;
	struct dir_entry	*cur;

	uint16_t		uid;
	uint16_t		gid;
	uint16_t		perms;
};

struct blocks_extent;
struct blocks_plain;

/* provides buffered IO similar to standard C I/O */
struct file {
	int fd;

	/* block info */
	union {
		struct blocks_extent	*extent;
		struct blocks_plain	*plain;
	} _union;
#define		pos_extent	_union.extent
#define		pos_plain	_union.plain
	uint32_t		block_count;
	uint32_t		block_pos;
	enum block_type		block_type;

	/* inode details */
	off_t			off_inode;
	off_t			sz;
	uint32_t		inode;
	uint16_t		uid;
	uint16_t		gid;
	uint16_t		perms;
	enum file_type		type;

	/* the last block to be read */
	uint8_t			buf[SZ_BLOCK];
	uint16_t		buf_pos;
	uint16_t		buf_sz;
	/* set when no more data to add to 'buf' */
	uint8_t			eof;
};

#endif
